/*
 * Qt5-UKUI's Library
 *
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Jing Tan <tanjing@kylinos.cn>
 *
 */

import QtQuick 2.12
import QtQuick.Controls 2.12
import QtQuick.Controls.impl 2.12
import QtQuick.Templates 2.12 as T
import org.ukui.qqc2style.private 1.0 as StylePrivate

T.RadioDelegate {
    id: control

    implicitWidth: Math.max(implicitBackgroundWidth + leftInset + rightInset,
                            implicitContentWidth + leftPadding + rightPadding)
    implicitHeight: Math.max(implicitBackgroundHeight + topInset + bottomInset,
                             implicitContentHeight + topPadding + bottomPadding,
                             implicitIndicatorHeight + topPadding + bottomPadding)
    hoverEnabled: true

    property alias indicatorColor: indicatorRec.color
    property alias childrenColor: childrenRec.color

    property var borderColor: control.checked ? (!control.enabled ? radiobutton.checked_disableIndicatorBorderColor :
                                                                    control.pressed ? radiobutton.checked_clickIndicatorBorderColor : control.hovered ?
                                                                    radiobutton.checked_hoverIndicatorBorderColor : radiobutton.checked_normalIndicatorBorderColor) :
                                                                   (!control.enabled ? radiobutton.disableIndicatorBorderColor : control.pressed ?
                                                                    radiobutton.clickIndicatorBorderColor : control.hovered ?
                                                                    radiobutton.hoverIndicatorBorderColor : radiobutton.normalIndicatorBorderColor)

    property var backColor: control.checked ? (!control.enabled ? radiobutton.checked_disableIndicatorColor :
                                                                  control.pressed ? radiobutton.checked_clickIndicatorColor : control.hovered ?
                                                                  radiobutton.checked_hoverIndicatorColor : radiobutton.checked_normalIndicatorColor) :
                                                                 (!control.enabled ? radiobutton.disableIndicatorColor : control.pressed ?
                                                                  radiobutton.clickIndicatorColor : control.hovered ?
                                                                  radiobutton.hoverIndicatorColor : radiobutton.normalIndicatorColor)

    property var childrenDtColor: !control.enabled ? radiobutton.checked_disableChildrenColor :
                                                   control.pressed ? radiobutton.checked_clickChildrenColor:
                                                                     control.hovered ? radiobutton.checked_hoverChildrenColor:
                                                                                       radiobutton.checked_normalChildrenColor

    ParseInterface{
        id: parseInterface
    }
    StylePrivate.UKUIRadioButton{
        id: radiobutton
    }

    StylePrivate.APPParameter{
        id: app
    }

    function getStartColor(dtcolor)
    {
        return parseInterface.pStartColor(dtcolor)
    }
    function getEndColor(dtcolor)
    {
        return parseInterface.pEndColor(dtcolor)
    }

    padding: 6
    spacing: radiobutton.space

    contentItem: IconLabelContent {
        id: iconLabel
        controlRoot: control
        anchors.left: control.left
        leftSpace: control.mirrored ? control.indicator.width + control.spacing : control.leftPadding
        //x: control.text ? (control.mirrored ? control.leftPadding : control.width - width - control.rightPadding) : control.leftPadding + (control.availableWidth - width) / 2
        //anchors.centerIn: control
        implicitWidth: iconLabel.adjustWidth(control.leftPadding + control.rightPadding) + indicator.implicitWidth + control.spacing

    }

    // keep in sync with RadioButton.qml (shared RadioIndicator.qml was removed for performance reasons)
    indicator: Rectangle {
        id: indicatorRec
        implicitWidth: radiobutton.indicatorWidth
        implicitHeight: implicitWidth

        x: control.mirrored ? control.leftPadding : control.width - width - control.rightPadding
        y: control.topPadding + (control.availableHeight - height) / 2

        radius: width / 2
        border.width: radiobutton.borderWidth
        border.color: getStartColor(borderColor)
        color: gradientRec.visible ? "transparent" : getStartColor(backColor)
        Rectangle{
            id: gradientRec
            anchors.fill: parent
            radius: parent.radius
            border.width: parent.border.width
            border.color: parent.border.color
            gradient:  Gradient {
                GradientStop { position: 0.0; color: getStartColor(backColor) }

                GradientStop { position: 1.0; color: getEndColor(backColor)}
            }
            visible: !parseInterface.isSolidPattern(backColor)
        }

        Rectangle {
            id:childrenRec
            width: radiobutton.childrenWidth
            height: width
            anchors.centerIn: parent
            radius: width / 2
            color: gradientChildrenRec.visible ? "transparent" : getStartColor(childrenDtColor)
            visible: control.checked

            Rectangle{
                id: gradientChildrenRec
                anchors.fill: parent
                radius: parent.radius
                border.width: parent.border.width
                border.color: parent.border.color
                gradient:  Gradient {
                    GradientStop { position: 0.0; color: getStartColor(childrenDtColor) }

                    GradientStop { position: 1.0; color: getEndColor(childrenDtColor)}
                }
                visible: !parseInterface.isSolidPattern(childrenDtColor)
            }
        }
    }

    background: Rectangle {
        implicitWidth: parent.implicitWidth
        implicitHeight: 36
        visible: control.down || control.highlighted
        color: control.down ? control.palette.midlight : control.palette.light
    }
}
